/*
 * Copyright (c) 2015 N. Billard.
 */
package com.massa.mrules.droolsfight.tests.mrules;

import java.io.IOException;

import com.massa.mrules.action.AbstractBasicAction;
import com.massa.mrules.action.IAction;
import com.massa.mrules.addon.PersistantProperties;
import com.massa.mrules.addon.PersistantProperty;
import com.massa.mrules.context.compile.ICompilationContext;
import com.massa.mrules.context.execute.IExecutionContext;
import com.massa.mrules.droolsfight.beans.MeasureBean;
import com.massa.mrules.droolsfight.beans.MeasureError;
import com.massa.mrules.exception.MActionExecutionException;
import com.massa.mrules.exception.MRuleValidationException;
import com.massa.mrules.executable.ExecutionResultState;
import com.massa.mrules.executable.IExecutionResult;
import com.massa.mrules.util.out.OuterWithLevel;
import com.massa.util.MBeanUtils;

/**
 * Action that adds an error on the current measure.
 */
@PersistantProperties(properties={
		@PersistantProperty(property="code"),
		@PersistantProperty(property="msg"),
		@PersistantProperty(property="blocking")
})
public class AddErrorAddon extends AbstractBasicAction {
	private static final long serialVersionUID = -6170279159087984571L;

	public static final String ADDERROR_ID = "ADDERROR";

	private String code;
	private String msg;
	private boolean blocking;

	public AddErrorAddon() {
		super();
	}

	@Override
	protected void compileExecutable(final ICompilationContext ctxt) throws MRuleValidationException {
	}

	/**
	 * @see IAction#execute(IExecutionContext)
	 */
	@Override
	protected IExecutionResult internalExecute(final IExecutionContext ctxt) throws MActionExecutionException {
		final MeasureBean data = (MeasureBean) ctxt.getWriteBase();
		data.addError(new MeasureError(this.code, this.msg, this.blocking));
		return ExecutionResultState.OK.COND_NA;
	}

	@Override
	public String getImplementationId() {
		return AddErrorAddon.ADDERROR_ID;
	}

	@Override
	public void toString(final OuterWithLevel outer) throws IOException {
		outer.write(this.getImplementationId()).write('(').write(this.code).write(", ").write(this.msg).write(", ").write(this.blocking).write(')');
	}

	@Override
	public AddErrorAddon clone() {
		return (AddErrorAddon) super.clone();
	}

	@Override
	public boolean equals(final Object o) {
		if (o == this) {
			return true;
		}
		if (o != null && this.getClass() == o.getClass()) {
			final AddErrorAddon oo = (AddErrorAddon) o;
			return MBeanUtils.eq(this.code, oo.code)
					&& MBeanUtils.eq(this.msg, oo.msg)
					&& MBeanUtils.eq(this.blocking, oo.blocking);
		}
		return false;
	}

	@Override
	public int hashCode() {
		return (this.blocking ? 3 : 1) * (MBeanUtils.computeHashCode(this.code) + MBeanUtils.computeHashCode(this.msg));
	}

	public String getCode() {
		return this.code;
	}

	public void setCode(final String code) {
		this.code = code;
	}

	public String getMsg() {
		return this.msg;
	}

	public void setMsg(final String msg) {
		this.msg = msg;
	}

	public boolean isBlocking() {
		return this.blocking;
	}

	public void setBlocking(final boolean blocking) {
		this.blocking = blocking;
	}

}
